/**
 * @file
 * Actionscript controller for pattern recognition
 */
 
 
package {
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.external.ExternalInterface;
	import flash.media.Camera;
	import flash.media.Video;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.ByteArray;
	
	import nl.demonsters.debugger.MonsterDebugger;
	
	import org.libspark.flartoolkit.core.FLARCode;
	import org.libspark.flartoolkit.core.param.FLARParam;
	import org.libspark.flartoolkit.core.raster.rgb.FLARRgbRaster_BitmapData;
	import org.libspark.flartoolkit.detector.FLARSingleMarkerDetector;
	import org.libspark.flartoolkit.pv3d.FLARCamera3D;

	// Destenation SWF file
	[SWF(width="640", height="480", frameRate="30", backgroundColor="#ccddff")]  

	public class AugmentRealityTrigger extends Sprite {
		[Embed(source="camera_para.dat", mimeType="application/octet-stream")]
		private var params:Class;
		
		[Embed(source="search.swf")]
		private var searchSprite:Class;
		
		[Embed(source="search_ready.swf")]
		private var searchReadySprite:Class;
		
		private var fparams:FLARParam;
		private var mpattern:FLARCode;
		private var vid:Video;
		private var cam:Camera;
		private var bmd:BitmapData;
		private var raster:FLARRgbRaster_BitmapData;
		private var detector:FLARSingleMarkerDetector;
		private var camera:FLARCamera3D;
		
		private var searchSpriteBgr:Sprite = new Sprite();
		private var searchSpriteBar:Sprite = new Sprite();
		
		private var hitPoint:int = 0;
		private var hitPointSuccess:int = 12; 
		
		//private var url:String = 'http://localhost/drupal_webcam_trigger/sites/all/modules/webcam_trigger/webcam_trigger.pat';
		
		public function AugmentRealityTrigger()	{
			var dd:MonsterDebugger = new MonsterDebugger(this);
			
			setupFLAR();
			setupCamera();
			setupBitmap();
			setupLoader();
		}
		
		public function setupLoader():void {
			searchSpriteBgr.addChild(new searchSprite() as Sprite);
			searchSpriteBgr.x = 6;
			searchSpriteBgr.y = 453;
			
			searchSpriteBar.graphics.beginFill(0x0073BA);
			searchSpriteBar.graphics.drawRoundRect(0, 0, 98, 17, 4, 4);
			searchSpriteBar.graphics.endFill();
			searchSpriteBar.x = 82;
			searchSpriteBar.y = 2;
			searchSpriteBar.width = 1;
			searchSpriteBgr.addChild(searchSpriteBar);
			
			addChild(searchSpriteBgr);
		}
		
		private function setupFLAR():void {
			fparams = new FLARParam();
			fparams.loadARParam(new params() as ByteArray);
			mpattern = new FLARCode(16, 16);
			
			var re:RegExp = /AugmentRealityTrigger.swf/;
			var load_url:String = this.loaderInfo.loaderURL;
			var pattern_url:String = load_url.replace(re, 'webcam_trigger.pat');
			var ur:URLRequest = new URLRequest(pattern_url);
			var ul:URLLoader = new URLLoader(ur);
			ul.load(ur);
			ul.addEventListener(Event.COMPLETE, function(event:Event):void {
				trace(event);
				mpattern.loadARPatt(event.target.data);
				addEventListener(Event.ENTER_FRAME, loop);
			});
			ul.addEventListener(IOErrorEvent.IO_ERROR, function(event:Event):void {
				trace(event);
			});
		}
		
		private function setupCamera():void {
			vid = new Video(640, 480);
			cam = Camera.getCamera();
			cam.setMode(640, 480, 30);
			vid.attachCamera(cam);
			addChild(vid);
		}
		
		private function setupBitmap():void {
			bmd = new BitmapData(640, 480);
			bmd.draw(vid);
			raster = new FLARRgbRaster_BitmapData(bmd);
			detector = new FLARSingleMarkerDetector(fparams, mpattern, 80);
		}
		
		private function loop(event:Event):void {
			bmd.draw(vid);
			
			try {
				if (detector.detectMarkerLite(raster, 80) && detector.getConfidence() > .5) {
					//trace('hit: ' + hitPoint);
					++hitPoint;
					if (hitPoint <= hitPointSuccess) {
						searchSpriteBar.width = hitPoint > 0 ? hitPoint / hitPointSuccess * 98 : 1;
						if (hitPoint == hitPointSuccess) {
							searchSpriteBgr.removeChildAt(0);
							searchSpriteBgr.removeChildAt(0);
							searchSpriteBgr.addChild(new searchReadySprite() as Sprite);
						}
					}
					
					if (hitPoint >= hitPointSuccess && ExternalInterface.available) {
						//trace('call javascript');
						ExternalInterface.call("flashTriggerCallback");
					}
				}
			} catch (error:Error) {
				//trace('Error: ' + error);
			}
		}
	}
}
